﻿using System;
using System.Diagnostics;

///////////////////////////////////////////////////////////////////////////////

// Da diese Klasse keinen expliziten Typkonstruktor definiert,
// markiert C# die Typdefinition in den Metadaten mit BeforeFieldInit.
internal sealed class BeforeFieldInit
{
   public static Int32 s_x = 123;
}

// Da diese Klasse explizit einen Typkonstruktor definiert,
// markiert C# die Typdefinition in den Metadaten nicht mit BeforeFieldInit.
internal sealed class Precise
{
   public static Int32 s_x;
   static Precise() { s_x = 123; }
}

///////////////////////////////////////////////////////////////////////////////

public sealed class Program
{
   public static void Main()
   {
      const Int32 iterations = 1000 * 1000 * 1000;
      PerfTest1(iterations);
      PerfTest2(iterations);
   }

   // Wenn der JIT-Compiler diese Methode übersetzt, wurden die Typkonstruktoren 
   // für die Klassen BeforeFieldInit und Precise NOCH NICHT ausgeführt.
   // Daher werden Aufrufe dieser Konstruktoren in den Code der Methode
   // eingebettet. Sie wird dadurch langsamer.
   private static void PerfTest1(Int32 iterations)
   {
      Stopwatch sw = Stopwatch.StartNew();
      for (Int32 x = 0; x < iterations; x++)
      {
         // Der JIT-Compiler stellt den Code zum Aufrufen des Typkonstruktors von
         // BeforeFieldInit voran, sodass er vor dem Beginn der Schleife ausgeführt wird.
         BeforeFieldInit.s_x = 1;
      }
      Console.WriteLine("PerfTest1: {0} BeforeFieldInit", sw.Elapsed);

      sw = Stopwatch.StartNew();
      for (Int32 x = 0; x < iterations; x++)
      {
         // Der JIT-Compiler generiert den Code zum Aufrufen des Typkonstruktors
         // von Precise so, dass er in jedem Schleifendurchlauf überprüft, ob
         // der Konstruktor aufgerufen werden muss.
         Precise.s_x = 1;
      }
      Console.WriteLine("PerfTest1: {0} Precise", sw.Elapsed);
   }

   // Wenn der JIT-Compiler diese Methode übersetzt, WURDEN die Typkonstruktoren 
   // für die Klassen BeforeFieldInit und Precise bereits ausgeführt.
   // Daher werden KEINE Aufrufe dieser Konstruktoren in den Code der Methode
   // eingebettet. Sie ist dadurch schneller.
   private static void PerfTest2(Int32 iterations)
   {
      Stopwatch sw = Stopwatch.StartNew();
      for (Int32 x = 0; x < iterations; x++)
      {
         BeforeFieldInit.s_x = 1;
      }
      Console.WriteLine("PerfTest2: {0} BeforeFieldInit", sw.Elapsed);

      sw = Stopwatch.StartNew();
      for (Int32 x = 0; x < iterations; x++)
      {
         Precise.s_x = 1;
      }
      Console.WriteLine("PerfTest2: {0} Precise", sw.Elapsed);
   }
}

////////////////////////////// Dateiende ////////////////////////////////////
