/******************************************************************************
Module:  DiskFullException.cs
Notices: Copyright (c) 2006 Jeffrey Richter
******************************************************************************/


using System;
using System.Text;
using System.IO;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using System.Security.Permissions;


///////////////////////////////////////////////////////////////////////////////


// Instanzen von DiskFullException sollen serialisierbar sein.
[Serializable]
public sealed class DiskFullException : Exception, ISerializable {
   // Ein privates Feld definieren.
   private String m_diskpath;

   // Eine schreibgeschtzte Eigenschaft, die das Feld zurckgibt.
   public String DiskPath { get { return m_diskpath; } }

   // Die ffentliche Eigenschaft Message berschreiben, sodass unser Feld
   // (sofern ihm ein Wert zugewiesen wurde) in der Meldung enthalten ist.
   public override String Message
   {
      get {
         if (m_diskpath == null) return base.Message;
         StringBuilder msg = new StringBuilder(base.Message);
         msg.AppendLine(" (DiskPath=" + m_diskpath +")");
         return msg.ToString();
      }
   }

   // Die drei ffentlichen Konstruktoren
   public DiskFullException() : base() {  }
   public DiskFullException(String message) : base(message) { }
   public DiskFullException(String message, Exception innerException)
      : base(message, innerException) { }

   // Zustzliche Konstruktoren zum Setzen des Feldes definieren.
   public DiskFullException(String message, String diskpath)
      : this(message) { m_diskpath = diskpath; }

   public DiskFullException(String message, String diskpath, Exception innerException) 
      : this(message, innerException) { m_diskpath = diskpath; }

   // Der eine Konstruktor fr die Deserialisierung.
   // Weil diese Klasse versiegelt ist, ist dieser Konstruktor privat.
   // Wre die Klasse nicht versiegelt, sollte der Konstruktor geschtzt sein.
   private DiskFullException(SerializationInfo info, StreamingContext context)
      : base(info, context) {
         // Jedes Feld deserialisieren.
      m_diskpath = info.GetString("DiskPath");
   }

   // Weil zumindest ein Feld definiert ist, mssen wir die Serialisierungsmethode definieren.
   [SecurityPermission(SecurityAction.Demand, SerializationFormatter = true)]
   public override void GetObjectData(
      SerializationInfo info, StreamingContext context) {
      // Felder dieses Typs serialisieren.
      info.AddValue("DiskPath", m_diskpath);

      // Der Basistyp soll seine Felder serialisieren.
      base.GetObjectData(info, context);
   }
}


///////////////////////////////////////////////////////////////////////////////


// Der folgende Code testet das Serialisieren der Ausnahme.
public sealed class Program {
   public static void Main() {
      // Ein DiskFullException-Objekt anlegen
      DiskFullException e =
         new DiskFullException("The disk volume is full", @"C:\");

      // Serialisieren des DiskFullException-Objekts
      FileStream fs = new FileStream("Test", FileMode.Create);
      IFormatter f = new BinaryFormatter();
      f.Serialize(fs, e);
      fs.Close();

      // Deserialisieren des DiskFullException-Objekts und berprfen seiner Felder
      fs = new FileStream("Test", FileMode.Open);
      e = (DiskFullException) f.Deserialize(fs);
      fs.Close();
      Console.WriteLine("Typ: {1}{0}DiskPath: {2}{0}Message: {3}",
         Environment.NewLine, e.GetType(), e.DiskPath, e.Message);
   }
}


//////////////////////////////// Dateiende //////////////////////////////////
