using System;
using System.Collections.Generic;

public static class Program
{
   public static void Main()
   {
      // Ein MailManager-Objekt anlegen.
      MailManager mm = new MailManager();

      // Ein Fax-Objekt anlegen.
      Fax f = new Fax(mm);

      // Da keine Garbage Collections stattgefunden haben, erhlt
      // das Fax-Objekt die Ereignisbenachrichtigung.
      mm.SimulateNewMail();

      // Eine Garbage Collection erzwingen.
      GC.Collect();

      // Da Garbage Collections stattgefunden haben, erhlt
      // das Fax-Objekt keine Ereignisbenachrichtigung.
      mm.SimulateNewMail();
   }
}

internal sealed class MailManager
{
   // Die Auflistung verwaltet den Satz von WeakReference-Objekten fr die Delegaten.
   private List<WeakReference> m_NewMailCallbacks = new List<WeakReference>();

   // Dieses Ereignis wird ausgelst, wenn neue E-Mail eintrifft.
   public event EventHandler NewMail
   {
      add
      {
         lock (m_NewMailCallbacks)
         {
            // Eine neue WeakReference als Wrapper um den bergebenen Delegaten erzeugen
            // und die WeakReference zu unserer Auflistung hinzufgen.
            m_NewMailCallbacks.Add(new WeakReference(value));
         }
      }

      remove
      {
         lock (m_NewMailCallbacks)
         {

            // In der Auflistung nach einem passenden Delegaten suchen und ihn entfernen.
            for (Int32 n = 0; n < m_NewMailCallbacks.Count; n++)
            {

               // Die WeakReference aus der Auflistung extrahieren.
               WeakReference wr = m_NewMailCallbacks[n];

               // Versuchen, den schwachen Verweis in einen starken Verweis umzuwandeln.
               EventHandler eh = (EventHandler)wr.Target;

               if (eh == null)
               {
                  // Das Objekt wurde vom Garbage Collector beseitigt.
                  // Diesen Eintrag aus der Auflistung lschen.
                  m_NewMailCallbacks.RemoveAt(n);
                  n--;        // Einen Eintrag zurckgehen.
                  continue;   // Den nchsten Eintrag versuchen.
               }

               // Das Objekt wurde nicht vom Garbage Collector beseitigt.
               // Entspricht der Delegat in der Auflistung dem bergebenen Delegaten?
               if ((eh.Target == value.Target) && (eh.Method == value.Method))
               {
                  // Ja, sie stimmen berein. Eintrag entfernen und zurckkehren.
                  m_NewMailCallbacks.RemoveAt(n);
                  break;
               }
            }
         }
      }
   }

   // Diese Methode knnen Sie aufrufen, um das Eintreffen einer neuen E-Mail zu simulieren.
   public void SimulateNewMail()
   {
      Console.WriteLine("Gleich wird das NewMail-Ereignis ausgelst.");
      OnNewMail(EventArgs.Empty);
   }

   // Diese Methode lst das NewMail-Ereignis aus.
   private void OnNewMail(EventArgs e)
   {
      lock (m_NewMailCallbacks)
      {

         // Unsere Auflistung durchsuchen und jeden Delegaten zurckrufen.
         for (Int32 n = 0; n < m_NewMailCallbacks.Count; n++)
         {

            // Die WeakReference aus der Auflistung extrahieren.
            WeakReference wr = m_NewMailCallbacks[n];

            // Versuchen, den schwachen Verweis in einen starken Verweis umzuwandeln.
            EventHandler eh = (EventHandler)wr.Target;

            if (eh == null)
            {
               // Das Objekt wurde vom Garbage Collector beseitigt.
               // Diesen Eintrag aus der Auflistung lschen.
               m_NewMailCallbacks.RemoveAt(n);
               n--;        // Einen Eintrag zurckgehen.
            }
            else
            {
               // Das Objekt wurde nicht vom Garbage Collector beseitigt.
               // Den Delegaten aufrufen.
               eh(this, e);
            }
         }
      }
   }
}

internal sealed class Fax
{
   // Beim Anlegen meldet sich das Objekt als Empfnger
   // fr das NewMail-Ereignis von MailManager an.
   public Fax(MailManager mm)
   {
      mm.NewMail += GotMail;
   }

   // Diese Methode wird aufgerufen, wenn das NewMail-Ereignis auftritt.
   public void GotMail(Object sender, EventArgs e)
   {
      // Nur zum Beweis, dass wir hier waren.
      Console.WriteLine("In Fax.GotMail");
   }
}

