using System;
using System.Reflection;
using System.Threading;
using System.Runtime.Remoting;

public static class Program
{
   public static void Main()
   {
      // Einen Verweis auf die AppDomain holen, in der der aufrufende Thread luft.
      AppDomain adCallingThreadDomain = Thread.GetDomain();

      // Jede AppDomain bekommt einen Namen zugewiesen, der fr das Debugging
      // ntzlich ist. Wir holen diesen Namen der AppDomain und zeigen ihn an.
      String callingDomainName = adCallingThreadDomain.FriendlyName;
      Console.WriteLine("Name der Standard-AppDomain={0}", callingDomainName);

      // Die Assembly in unserer AppDomain holen und anzeigen, die
      // die Methode 'Main' enthlt.
      String exeAssembly = Assembly.GetEntryAssembly().FullName;
      Console.WriteLine("Assembly mit Main={0}", exeAssembly);

      // Eine lokale Variable definieren, die auf eine AppDomain verweisen kann.
      AppDomain ad2 = null;

      // DEMO 1: Kommunikation ber AppDomains hinweg mit Marshal-by-Reference.
      Console.WriteLine("{0}Demo 1: Marshal-by-Reference", Environment.NewLine);

      // Eine neue AppDomain anlegen (Sicherheit und Konfiguration
      // entsprechen der aktuellen AppDomain)
      ad2 = AppDomain.CreateDomain("AD 2", null, null);

      // Unsere Assembly in die neue AppDomain laden, ein Objekt anlegen, und es
      // ber Marshalling an unsere AppDomain zurckgeben  (in Wirklichkeit
      // bekommen wir einen Verweis auf einen Proxy).
      MarshalByRefType mbrt = (MarshalByRefType)
         ad2.CreateInstanceAndUnwrap(exeAssembly, "MarshalByRefType");
      Type t = mbrt.GetType();

      // Beweisen, dass wir einen Verweis auf ein Proxyobjekt haben.
      Console.WriteLine("Ist ein Proxy={0}", RemotingServices.IsTransparentProxy(mbrt));

      // Hier sieht es aus, als wrden wir einen Methode fr eine
      // MarshalByRefType-Instanz aufrufen, aber das ist nicht der Fall.
      // Wir rufen eine Methode fr eine Instanz eines Proxytyps auf.
      // Der Proxy fhrt den Thread in die AppDomain ber, die das Objekt
      // besitzt, und ruft diese Methode fr das echte Objekt auf.
      mbrt.SomeMethod(callingDomainName);

      // Die neue AppDomain entladen.
      AppDomain.Unload(ad2);
      // mbrt verweist auf ein gltiges Proxyobjekt.
      // Dieser Proxy verweist jetzt auf eine ungltige AppDomain.

      try
      {
         // Wir rufen eine Methode fr das Proxytypobjekt auf.
         // Die AppDomain ist ungltig, es wird eine Ausnahme ausgelst.
         mbrt.SomeMethod(callingDomainName);
         Console.WriteLine("Aufruf erfolgreich.");
      }
      catch (AppDomainUnloadedException)
      {
         Console.WriteLine("Aufruf fehlgeschlagen.");
      }

      // DEMO 2: Kommunikation ber AppDomains hinweg mit Marshal-by-Value.
      Console.WriteLine("{0}Demo 2: Marshal-by-Value", Environment.NewLine);

      // Eine neue AppDomain anlegen (Sicherheit und Konfiguration
      // entsprechen der aktuellen AppDomain)
      ad2 = AppDomain.CreateDomain("AD 2", null, null);

      // Unsere Assembly in die neue AppDomain laden, ein Objekt anlegen, und es
      // ber Marshalling an unsere AppDomain zurckgeben  (in Wirklichkeit
      // bekommen wir eine Kopie des Objekts mit demselben Zustand).
      MarshalByValType mbvt = (MarshalByValType)
         ad2.CreateInstanceAndUnwrap(exeAssembly, "MarshalByValType");

      // Beweisen, dass wir KEINEN Verweis auf ein Proxyobjekt haben.
      Console.WriteLine("Ist ein Proxy={0}", RemotingServices.IsTransparentProxy(mbvt));

      // Hier sieht es aus, als wrden wir eine Methode fr ein
      // MarshalByValType-Objekt aufrufen, und genau das tun wir auch.
      mbvt.SomeMethod(callingDomainName);

      // Die neue AppDomain entladen
      AppDomain.Unload(ad2);
      // mbvt verweist auf ein gltiges Objekt. Es hat keine Auswirkung
      // auf dieses Objekt, dass wir die AppDomain entladen haben.

      try
      {
         // Wir rufen eine Methode fr das Objekt auf.
         // Es wird keine Ausnahme ausgelst.
         mbvt.SomeMethod(callingDomainName);
         Console.WriteLine("Aufruf erfolgreich.");
      }
      catch (AppDomainUnloadedException)
      {
         Console.WriteLine("Aufruf fehlgeschlagen.");
      }

      // DEMO 3: Kommunikation ber AppDomains hinweg ohne Marshalling-Typen.
      Console.WriteLine("{0}Demo 3: Kein Marshalling-Typ", Environment.NewLine);

      // Eine neue AppDomain anlegen (Sicherheit und Konfiguration
      // entsprechen der aktuellen AppDomain).
      ad2 = AppDomain.CreateDomain("AD 2", null, null);

      // Unsere Assembly in die neue AppDomain laden, ein Objekt anlegen, und
      // versuchen, es ber Marshalling an unsere AppDomain zurckzugeben
      // (es wird eine Ausnahme ausgelst).
      NotMarshalableType nmt = (NotMarshalableType)
         ad2.CreateInstanceAndUnwrap(exeAssembly, "NotMarshalableType");
      // Hierher kommen wir gar nicht...
   }
}

// Instanzen knnen mithilfe von Marshal-by-Reference
// ber AppDomain-Grenzen hinweg benutzt werden.
public class MarshalByRefType : MarshalByRefObject
{
   DateTime creation = DateTime.Now;

   public MarshalByRefType()
   {
      Console.WriteLine("{0} ctor luft in {1}",
         this.GetType().ToString(), Thread.GetDomain().FriendlyName);
   }

   public void SomeMethod(String callingDomainName)
   {
      Console.WriteLine("Aufruf aus '{0}' in '{1}'.",
         callingDomainName, Thread.GetDomain().FriendlyName);
   }
}

// Instanzen knnen mithilfe von Marshal-by-Value
// ber AppDomain-Grenzen hinweg benutzt werden.
[Serializable]
public class MarshalByValType : Object
{
   DateTime creation = DateTime.Now;

   public MarshalByValType()
   {
      Console.WriteLine("{0} ctor luft in {1}",
         this.GetType().ToString(), Thread.GetDomain().FriendlyName);
   }

   public void SomeMethod(String callingDomainName)
   {
      Console.WriteLine("Aufruf aus '{0}' in '{1}'.",
         callingDomainName, Thread.GetDomain().FriendlyName);
   }
}

// Instanzen knnen nicht ber AppDomain-Grenzen hinweg benutzt werden.
// [Serializable]
public class NotMarshalableType : Object
{
   DateTime creation = DateTime.Now;

   public NotMarshalableType()
   {
      Console.WriteLine("{0} ctor luft in {1}",
         this.GetType().ToString(), Thread.GetDomain().FriendlyName);
   }

   public void SomeMethod(String callingDomainName)
   {
      Console.WriteLine("Aufruf aus '{0}' in '{1}'.",
         callingDomainName, Thread.GetDomain().FriendlyName);
   }
}
